window.onload = function() {

  var settings = Drupal.settings.borealis_ri.sizes;
  var settingsArray = sortObject(settings);
  var settingArrayLength = settingsArray.length;

  var sizeArray = new Array();
  var styleArray = new Array();
  for (i = 0; i < settingArrayLength; i++) {
    sizeArray.push(settingsArray[i].value);
    styleArray.push(settingsArray[i].key);
  }

  Drupal.settings.borealis_ri.sizeArray = sizeArray;
  Drupal.settings.borealis_ri.styleArray = styleArray;

  // Call borealisImageRespond to initialize the images!
  borealisImageRespond();
};

// When the screen is resized, check again!
window.onresize = function() {
  borealisImageRespond();
}

//////////////////////////////
// Add indexOf Polyfill
//
// indexOf is new to ECMA-262 standard and as such, if it doesn't exist, we need to polyfill for it.
// Polyfill from https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Array/indexOf#Compatibility
//////////////////////////////
if (!Array.prototype.indexOf) {
  Array.prototype.indexOf = function (searchElement /*, fromIndex */ ) {
    "use strict";
    if (this == null) {
      throw new TypeError();
    }
    var t = Object(this);
    var len = t.length >>> 0;
    if (len === 0) {
      return -1;
    }
    var n = 0;
    if (arguments.length > 1) {
      n = Number(arguments[1]);
      if (n != n) { // shortcut for verifying if it's NaN
          n = 0;
      } else if (n != 0 && n != Infinity && n != -Infinity) {
          n = (n > 0 || -1) * Math.floor(Math.abs(n));
      }
    }
    if (n >= len) {
      return -1;
    }
    var k = n >= 0 ? n : Math.max(len - Math.abs(n), 0);
    for (; k < len; k++) {
      if (k in t && t[k] === searchElement) {
          return k;
      }
    }
    return -1;
  }
}


//////////////////////////////
// Add getComputedStyle Polyfill
//
// indexOf is new to ECMA-262 standard and as such, if it doesn't exist, we need to polyfill for it.
// Polyfill from http://snipplr.com/view/13523/
//////////////////////////////
if (!window.getComputedStyle) {
  window.getComputedStyle = function(el, pseudo) {
    this.el = el;
    this.getPropertyValue = function(prop) {
      var re = /(\-([a-z]){1})/g;
      if (prop == 'float') prop = 'styleFloat';
      if (re.test(prop)) {
        prop = prop.replace(re, function () {
          return arguments[2].toUpperCase();
        });
      }
      return el.currentStyle[prop] ? el.currentStyle[prop] : null;
    }
    return this;
  }
}

//////////////////////////////
// Borealis Image Respond
//
// Loop over each image, and change their source depending on parent width
//////////////////////////////
function borealisImageRespond() {

  var sizeArray = Drupal.settings.borealis_ri.sizeArray;
  var arraySize = sizeArray.length;
  var swapped = false;

  var images = document.querySelectorAll('img.borealis');
  var imageLength = images.length;

  for (var i = 0; i < imageLength; i++) {
    var image = images[i];
    var width = blockParentWidth(image);

    if (width > 0) {
      var slfor = arraySize - 1;
      swapped = false;
      for (j = slfor; j >= 0; j--) {

        // If the width is less than or equal to the smallest size, use the smallest size
        if (width <= sizeArray[0]) {
          borealisImageSwap(image, -1);
          j = -1;
        }
        else {
          if (width >= sizeArray[slfor + 1]) {
            borealisImageSwap(image, slfor, true);
            j = -1;
          }
          else if (width >= sizeArray[j]) {
            borealisImageSwap(image, j);
            j = -1;
          }
        }
      } // End Size Loop
    } //End Width > 0 Check
  } // End Image Loop
}

//////////////////////////////
// Borealis Image Swap
//
// Sets the appropriate image style based off of Device Pixel Ratio
//////////////////////////////
function borealisImageSwap(img, index, bypass) {
  // Increase index by one to get the next highest size
  index++;

  // Set Bypass
  bypass = (typeof bypass === "undefined") ? false : bypass;

  // Get current image style
  var imgStyle = img.getAttribute('data-borealis-style') ? img.getAttribute('data-borealis-style') : Drupal.settings.borealis_ri.styleArray[0];

  var styleArray = Drupal.settings.borealis_ri.styleArray;

  // Get Device Pixel Ratio
  var dpr = (window.devicePixelRatio !== undefined) ? window.devicePixelRatio : 1;

  // Create Style variable
  var style = '';

  // Bypass for the index size passed regardless of DPR
  if (bypass) {
    dpr = 1;
  }

  // If we've got a HighDPI device,
  if (dpr > 1.5 && dpr <= 2.5) {
    style = styleArray[index + 1];
  }
  else {
    style = styleArray[index];
  }

  // Only swap the image if the required image is larger
  if (styleArray.indexOf(style) > styleArray.indexOf(imgStyle) || img.getAttribute('src') == '//:0?placeholder') {
    var src = img.getAttribute('data-borealis-' + style);

    img.setAttribute('src', src);
    img.setAttribute('data-borealis-style', style);
  }
}

//////////////////////////////
// Returns the width of the nearest non-inline parent.
//////////////////////////////
function blockParentWidth(element) {
  var parentNode = element.parentNode;
  var parentDisplay = getComputedStyle(parentNode).display;

  if (parentDisplay.indexOf('inline') >= 0 && parentDisplay.indexOf('block') < 0) {
    return blockParentWidth(parentNode);
  }
  else {
    return parseInt(getComputedStyle(parentNode).width);
  }
}

//////////////////////////////
// Sorts an object into an array by value. http://stackoverflow.com/questions/1069666/sorting-javascript-object-by-property-value
//////////////////////////////
function sortObject(obj) {
    var arr = new Array();
    for (attr in obj) {
        arr.push({
            'key': attr,
            'value': obj[attr]
        });
    }
    arr.sort(function(a, b) { return a['value'] - b['value'] });
    return arr; // returns array
}